﻿-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- Variables.lua
-- Variables, constants and static popup tables.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];
local L = LibStub("AceLocale-3.0"):GetLocale("BrokerStats");

-- ********************************************************************************
-- Variables
-- ********************************************************************************

-- Others Data Broker objects table
A.dbObjects = {};

-- Player spec
A.playerSpec = "primary";

-- ********************************************************************************
-- Constants
-- ********************************************************************************

-- AddOn version
A.version = GetAddOnMetadata("Broker_Stats", "Version");

-- Text colors
A.colorRed = "|cffff3333";
A.colorGreen = "|cff33ff99";
A.colorWhite = "|cffffffff";
A.colorReset = "|r";

-- Dropdown menu align right
A.ddmAlignRight = "     ";

-- Build in stats table
A.buildInStats =
{
    [L["Speed"]] =
    {
        type = "misc",
        pre = L["Speed: "],
        post = "%",
        round = 0,
        calc = [[
return GetUnitSpeed("player") / BASE_MOVEMENT_SPEED * 100;
        ]]
    },
    [L["Avoidance"]] =
    {
        type = "tank",
        pre = L["Av: "],
        post = "%",
        round = 2,
        calc = [[
return GetDodgeChance() + GetBlockChance() + GetParryChance() + 5;
        ]]
    },
    [L["Attack Power"]] =
    {
        type = "melee",
        pre = L["AP: "],
        calc = [[
local base, posBuff, negBuff = UnitAttackPower("player");

return base + posBuff + negBuff;
        ]]
    },
    [L["Armor"]] =
    {
        type = "tank",
        pre = L["Armor: "],
        calc = [[
local _, effectiveArmor = UnitArmor("player");

return effectiveArmor;
        ]]
    },
    [L["Attack Speed"]] =
    {
        type = "melee",
        pre = L["Aspd: "],
        multiple = 1,
        separator = "/",
        round = 2,
        calc = [[
local mainSpeed, offSpeed = UnitAttackSpeed("player");

return mainSpeed, offSpeed;
        ]]
    },
    [L["Weapon Damage"]] =
    {
        type = "melee",
        pre = L["Wd: "],
        multiple = 1,
        separator = "/",
        calc = [[
local mainSpeed, offSpeed = UnitAttackSpeed("player");
local minDamage, maxDamage, minOffHandDamage, maxOffHandDamage, physicalBonusPos, physicalBonusNeg, percent = UnitDamage("player");

minDamage = (minDamage / percent) - physicalBonusPos - physicalBonusNeg;
maxDamage = (maxDamage / percent) - physicalBonusPos - physicalBonusNeg;

if ( offSpeed ) then
    minOffHandDamage = (minOffHandDamage / percent) - physicalBonusPos - physicalBonusNeg;
    maxOffHandDamage = (maxOffHandDamage / percent) - physicalBonusPos - physicalBonusNeg;

    return max(floor(minDamage), 1).."-"..max(ceil(maxDamage), 1), max(floor(minOffHandDamage), 1).."-"..max(ceil(maxOffHandDamage), 1);
else
    return max(floor(minDamage), 1).."-"..max(ceil(maxDamage), 1);
end
        ]]
    },
    [L["Strength"]] =
    {
        type = "misc",
        pre = L["Str: "],
        calc = [[
local _, stat = UnitStat("player", 1);

return stat;
        ]]
    },
    [L["Agility"]] =
    {
        type = "misc",
        pre = L["Agi: "],
        calc = [[
local _, stat = UnitStat("player", 2);

return stat;
        ]]
    },
    [L["Stamina"]] =
    {
        type = "misc",
        pre = L["Sta: "],
        calc = [[
local _, stat = UnitStat("player", 3);

return stat;
        ]]
    },
    [L["Intellect"]] =
    {
        type = "misc",
        pre = L["Int: "],
        calc = [[
local _, stat = UnitStat("player", 4);

return stat;
        ]]
    },
    [L["Spirit"]] =
    {
        type = "misc",
        pre = L["Spi: "],
        calc = [[
local _, stat = UnitStat("player", 5);

return stat;
        ]]
    },
    [L["Ranged Attack Power"]] =
    {
        type = "ranged",
        pre = L["RAP: "],
        calc = [[
local base, posBuff, negBuff = UnitRangedAttackPower("player");

return base + posBuff + negBuff;
        ]]
    },
    [L["Ranged Weapon Damage"]] =
    {
        type = "ranged",
        pre = L["Rwd: "],
        calc = [[
local _, minDamage, maxDamage, physicalBonusPos, physicalBonusNeg, percent = UnitRangedDamage("player");

minDamage = (minDamage / percent) - physicalBonusPos - physicalBonusNeg;
maxDamage = (maxDamage / percent) - physicalBonusPos - physicalBonusNeg;

return max(floor(minDamage), 1).."-"..max(ceil(maxDamage), 1);
        ]]
    },
    [L["Ranged Weapon Speed"]] =
    {
        type = "ranged",
        pre = L["Rspd: "],
        round = 2,
        calc = [[
return UnitRangedDamage("player");
        ]]
    },
    [L["Armor Penetration"]] =
    {
        type = "misc",
        pre = L["ArPen: "],
        post = "%",
        round = 2,
        calc = [[
return GetArmorPenetration();
        ]]
    },
    [L["Critical Chance"]] =
    {
        type = "melee",
        pre = L["CC: "],
        post = "%",
        round = 2,
        calc = [[
return GetCritChance();
        ]]
    },
    [L["Ranged Critical Chance"]] =
    {
        type = "ranged",
        pre = L["RCC: "],
        post = "%",
        round = 2,
        calc = [[
return GetRangedCritChance();
        ]]
    },
    [L["Global Spell Power"]] =
    {
        type = "spell",
        pre = L["SP: "],
        calc = [[
local spellPower = 0;

for i=2,7 do
    local s = GetSpellBonusDamage(i);

    if ( s > spellPower ) then spellPower = s; end
end

return spellPower;
        ]]
    },
    [L["Holy Spell Power"]] =
    {
        type = "spell",
        pre = L["Hsp: "],
        calc = [[
return GetSpellBonusDamage(2);
        ]]
    },
    [L["Fire Spell Power"]] =
    {
        type = "spell",
        pre = L["Fsp: "],
        calc = [[
return GetSpellBonusDamage(3);
        ]]
    },
    [L["Nature Spell Power"]] =
    {
        type = "spell",
        pre = L["Nsp: "],
        calc = [[
return GetSpellBonusDamage(4);
        ]]
    },
    [L["Frost Spell Power"]] =
    {
        type = "spell",
        pre = L["Fsp: "],
        calc = [[
return GetSpellBonusDamage(5);
        ]]
    },
    [L["Shadow Spell Power"]] =
    {
        type = "spell",
        pre = L["Ssp: "],
        calc = [[
return GetSpellBonusDamage(6);
        ]]
    },
    [L["Arcane Spell Power"]] =
    {
        type = "spell",
        pre = L["Asp: "],
        calc = [[
return GetSpellBonusDamage(7);
        ]]
    },
    [L["Global Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["SCC: "],
        post = "%",
        round = 2,
        calc = [[
local spellCrit = 0;

for i=2,7 do
    local c = GetSpellCritChance(i);

    if ( c > spellCrit ) then spellCrit = c; end
end

return spellCrit;
        ]]
    },
    [L["Holy Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Hscc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(2);
        ]]
    },
    [L["Fire Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Fscc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(3);
        ]]
    },
    [L["Nature Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Nscc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(4);
        ]]
    },
    [L["Frost Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Fscc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(5);
        ]]
    },
    [L["Shadow Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Sscc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(6);
        ]]
    },
    [L["Arcane Spell Critical Chance"]] =
    {
        type = "spell",
        pre = L["Ascc: "],
        post = "%",
        round = 2,
        calc = [[
return GetSpellCritChance(7);
        ]]
    },
    [L["Spell Penetration"]] =
    {
        type = "spell",
        pre = L["Spen: "],
        calc = [[
return GetSpellPenetration();
        ]]
    },
    [L["Hit Chance"]] =
    {
        type = "melee",
        pre = L["Hit: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HIT_MELEE) + GetHitModifier();
        ]]
    },
    [L["Ranged Hit Chance"]] =
    {
        type = "ranged",
        pre = L["RaHit: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HIT_RANGED) + GetHitModifier();
        ]]
    },
    [L["Spell Hit Chance"]] =
    {
        type = "spell",
        pre = L["SpHit: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HIT_SPELL) + GetSpellHitModifier();
        ]]
    },
    [L["Resilience"]] =
    {
        type = "misc",
        pre = L["Resi: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN);
        ]]
    },
    [L["Resilience Rating"]] =
    {
        type = "misc",
        pre = L["Resi: "],
        calc = [[
return GetCombatRating(COMBAT_RATING_RESILIENCE_PLAYER_DAMAGE_TAKEN);
        ]]
    },
    [L["Haste"]] =
    {
        type = "melee",
        pre = L["Haste: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HASTE_MELEE);
        ]]
    },
    [L["Haste Rating"]] =
    {
        type = "melee",
        pre = L["Haste: "],
        calc = [[
return GetCombatRating(CR_HASTE_MELEE);
        ]]
    },
    [L["Ranged Haste"]] =
    {
        type = "ranged",
        pre = L["RaHaste: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HASTE_RANGED);
        ]]
    },
    [L["Ranged Haste Rating"]] =
    {
        type = "ranged",
        pre = L["RaHaste: "],
        calc = [[
return GetCombatRating(CR_HASTE_RANGED);
        ]]
    },
    [L["Spell Haste"]] =
    {
        type = "spell",
        pre = L["SpHaste: "],
        post = "%",
        round = 2,
        calc = [[
return GetCombatRatingBonus(CR_HASTE_SPELL);
        ]]
    },
    [L["Spell Haste Rating"]] =
    {
        type = "spell",
        pre = L["SpHaste: "],
        calc = [[
return GetCombatRating(CR_HASTE_SPELL);
        ]]
    },
    [L["Mastery"]] =
    {
        type = "misc",
        pre = L["Mast: "],
        round = 2,
        calc = [[
return GetMastery();
        ]]
    },
    [L["Mastery Rating"]] =
    {
        type = "misc",
        pre = L["Mast: "],
        calc = [[
return GetCombatRating(CR_MASTERY);
        ]]
    },
    [L["Expertise"]] =
    {
        type = "melee",
        pre = L["Exp: "],
        multiple = 1,
        separator = "/",
        calc = [[
local mainSpeed, offSpeed = UnitAttackSpeed("player");
local expertise, offhandExpertise = GetExpertise();

if ( offSpeed ) then
    return expertise, offhandExpertise;
else
    return expertise;
end
        ]]
    },
    [L["Expertise Percent"]] =
    {
        type = "melee",
        pre = L["Exp: "],
        post = "%",
        multiple = 1,
        separator = "/",
        calc = [[
local mainSpeed, offSpeed = UnitAttackSpeed("player");
local expertise, offhandExpertise = GetExpertisePercent();

if ( offSpeed ) then
    return expertise, offhandExpertise;
else
    return expertise;
end
        ]]
    },
    [L["Healing Power"]] =
    {
        type = "spell",
        pre = L["Hpw: "],
        calc = [[
return GetSpellBonusHealing();
        ]]
    },
    [L["Mana regen"]] =
    {
        type = "spell",
        pre = L["R: "],
        round = 0,
        calc = [[
local regen = GetManaRegen();

return regen * 5;
        ]]
    },
    [L["Combat mana regen"]] =
    {
        type = "spell",
        pre = L["Rc: "],
        round = 0,
        calc = [[
local _,regen = GetManaRegen();

return regen * 5;
        ]]
    }
};

-- ********************************************************************************
-- StaticPopup
-- ********************************************************************************

-- Sending a custom stat
StaticPopupDialogs["BrokerStatsSendCustom"] =
{
    text = L["Enter the player name:"],
    button1 = L["Send"],
    button2 = L["Cancel"],
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    enterClicksFirstButton = true,
    OnShow = function(self) self.button1:Disable(); end,
    EditBoxOnTextChanged = function (self) self:GetParent().button1:Enable(); end,
    EditBoxOnEnterPressed = function(self)
        A:SendStat(self:GetParent().editBox:GetText(), self:GetParent().customStat);
        self:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function(self) self:GetParent():Hide(); end,
    OnAccept = function(self) A:SendStat(self.editBox:GetText(), self.customStat); end
};

-- Receiving a custom stat
StaticPopupDialogs["BrokerStatsReceiveCustom"] =
{
    text = L["Received custom stat \"%s\" from \"%s\"."],
    button1 = L["Accept"],
    button2 = L["Cancel"],
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    enterClicksFirstButton = true,
    OnAccept = function(self)
        local customTable = loadstring("return "..self.customText);
        customTable = customTable();

        if ( self.customName ) then
            A.db.global.custom[self.customName] = customTable;
            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");
        else
            A:Message(L["The name provided was empty."], 1, 1);
        end
    end
};

-- Receiving a custom stat with an existing name
StaticPopupDialogs["BrokerStatsReceiveCustomNameExists"] =
{
    text = L["Received custom stat \"%s\" from \"%s\".\n\nYou already got a custom stat with that name, enter a new name:"],
    button1 = L["Accept"],
    button2 = L["Cancel"],
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    enterClicksFirstButton = true,
    OnShow = function(self) self.button1:Disable(); end,
    EditBoxOnTextChanged = function (self) self:GetParent().button1:Enable(); end,
    EditBoxOnEnterPressed = function(self)
        local name = strtrim(self:GetParent().editBox:GetText());

        if ( not name or name == "" ) then
            A:Message(L["The name provided was empty."], 1, 1);
            return;
        end

        if ( A.db.global.custom[name] ) then
            A:Message(L["The custom stat %s already exists."]:format(name), 1, 1);
            self:GetParent():Hide();
        else
            local customTable = loadstring("return "..self:GetParent().customText);
            customTable = customTable();

            A.db.global.custom[name] = customTable;
            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");

            self:GetParent():Hide();
        end
	end,
	EditBoxOnEscapePressed = function(self) self:GetParent():Hide(); end,
    OnAccept = function(self)
        local name = strtrim(self.editBox:GetText());

        if ( not name or name == "" ) then
            A:Message(L["The name provided was empty."], 1, 1);
            return;
        end

        if ( A.db.global.custom[name] ) then
            A:Message(L["The custom stat %s already exists."]:format(name), 1, 1);
        else
            local customTable = loadstring("return "..self.customText);
            customTable = customTable();

            A.db.global.custom[name] = customTable;
            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");
        end
    end
};

-- Adding a custom stat with an existing name
StaticPopupDialogs["BrokerStatsAddCustomNameExists"] =
{
    text = L["You already got a custom stat with that name, enter a new name:"],
    button1 = L["Accept"],
    button2 = L["Cancel"],
    hasEditBox = true,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    enterClicksFirstButton = true,
    OnShow = function(self) self.button1:Disable(); end,
    EditBoxOnTextChanged = function (self) self:GetParent().button1:Enable(); end,
    EditBoxOnEnterPressed = function(self)
        local name = strtrim(self:GetParent().editBox:GetText());

        if ( not name or name == "" ) then
            A:Message(L["The name provided was empty."], 1, 1);
            return;
        end

        if ( A.db.global.custom[name] ) then
            A:Message(L["The custom stat %s already exists."]:format(name), 1, 1);
            self:GetParent():Hide();
        else
            local customTable = loadstring("return "..self:GetParent().customText);
            customTable = customTable();

            A.db.global.custom[name] = customTable;
            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");

            self:GetParent():Hide();
        end
	end,
	EditBoxOnEscapePressed = function(self) self:GetParent():Hide(); end,
    OnAccept = function(self)
        local name = strtrim(self.editBox:GetText());

        if ( not name or name == "" ) then
            A:Message(L["The name provided was empty."], 1, 1);
            return;
        end

        if ( A.db.global.custom[name] ) then
            A:Message(L["The custom stat %s already exists."]:format(name), 1, 1);
        else
            local customTable = loadstring("return "..self.customText);
            customTable = customTable();

            A.db.global.custom[name] = customTable;
            LibStub("AceConfigRegistry-3.0"):NotifyChange("BrokerStatsConfig");
        end
    end
};
